# Copyright (C) 2009 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA


import errno
import re
from subprocess import Popen, PIPE

from bzrlib import(
    errors,
    osutils,
    ui,
)


body_template = """\
= Summary =

== Proposed fix ==

== Pre-implementation notes ==

== Implementation details ==

== Tests ==

== Demo and Q/A ==
"""


def body_callback(params):
    """Callback for use with the merge_request_body hook.

    :param params: The hook parameters for a merge_request_body hook.
    :return: The desired body.
    """
    if params.body is not None:
        return params.body

    def modified_files_callback():
        base_tree = params.tree.branch.repository.revision_tree(
            params.directive.base_revision_id)
        target_tree = params.tree.branch.repository.revision_tree(
            params.directive.revision_id)
        return modified_files(base_tree, target_tree)
    return get_body(params.tree, params.directive.target_branch,
                    modified_files_callback, params.body)


def get_body(tree, target_branch, modified_files_callback, old_body):
    m = re.match('lp:(~launchpad-pqm/)?launchpad(/(db-)?devel)?',
                 target_branch)
    if m:
        raise errors.BzrCommandError("Please avoid using lp: URLs to specify "
            "the submit branch as PQM does not support them. Instead use e.g. "
            "bzr+ssh://bazaar.launchpad.net/%slaunchpad%s" %
                (m.group(1) or "~launchpad-pqm/", m.group(2) or "/devel"))
    if not re.match('bzr\+ssh://bazaar.launchpad.net/~launchpad-pqm'
                    '/launchpad/(db-)?devel', target_branch):
        return old_body
    if tree is None:
        lint = ''
    else:
        lint = get_lint(tree, modified_files_callback())
    return body_template + lint


def modified_files(old_tree, new_tree):
    for f, (op, path), c, v, p, n, (ok, k), e in new_tree.iter_changes(
        old_tree):
        if c and k == 'file':
            yield str(path)


def get_lint(tree, modified_files):
    """Calculate the lint for a merge directive.

    :param tree: The tree to run the lint script in.
    :param directive: The MergeDirective to determine the changes from.
    :return: The output of lint.sh, or '', or '(No files to lint)'
    """
    if tree is None:
        return ''
    pb = ui.ui_factory.nested_progress_bar()
    try:
        pb.update('Calculating lint', 0, 2)
        args = ['utilities/lint.sh']
        args.extend(modified_files)
        if len(args) == 1:
            return '\n\n(No files to lint)'
        pb.update('Calculating lint', 1, 2)
        try:
            lint_proc = Popen(args, stdout=PIPE, cwd=tree.basedir)
        except OSError, e:
            if e.errno != errno.ENOENT:
                raise
            args[0] = tree.abspath('bin/lint.sh')
            try:
                lint_proc = Popen(args, stdout=PIPE, cwd=tree.basedir)
            except OSError, e:
                if e.errno == errno.ENOENT:
                    raise errors.BzrCommandError('Cannot find %s' % args[0])
                elif e.errno == errno.EACCES:
                    raise errors.BzrCommandError(
                    '%s does not have execute permissions.' % args[0])
                else:
                    raise
        stdout, stderr = lint_proc.communicate()
    finally:
        pb.finished()
    return '\n\n' + stdout
